from base64 import b64decode
import json
import re

from .. import strategy


@strategy.register("db_v1_tariff")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(decode(dp_value))
    return status_key, status_value



class TimerVO:
    def __init__(self, start_time, end_time):
        self.start_time = start_time
        self.end_time = end_time


class DBV1TariffPeriodBO:
    def __init__(self):
        self.monday = []
        self.tuesday = []
        self.wednesday = []
        self.thursday = []
        self.friday = []
        self.saturday = []
        self.sunday = []


def decode(input_string):
    return b64decode(input_string)


def convert_value(str_input):
    datelist = average_str(str_input, 12)
    if len(datelist) != 7:
        return ""
    dbv1_tariff_period = DBV1TariffPeriodBO()

    for i in range(7):
        timer_list = []
        item = datelist[i]
        if item != "0000000000000000":
            day_list = average_str(item, 4)
            for day_item in day_list:
                start_time = str(hex2decimal(day_item[:2]))
                end_time = str(hex2decimal(day_item[2:4]))
                timer_list.append(TimerVO(start_time, end_time))

        if i == 0:
            dbv1_tariff_period.monday = timer_list
        elif i == 1:
            dbv1_tariff_period.tuesday = timer_list
        elif i == 2:
            dbv1_tariff_period.wednesday = timer_list
        elif i == 3:
            dbv1_tariff_period.thursday = timer_list
        elif i == 4:
            dbv1_tariff_period.friday = timer_list
        elif i == 5:
            dbv1_tariff_period.saturday = timer_list
        elif i == 6:
            dbv1_tariff_period.sunday = timer_list

    return json.dumps(dbv1_tariff_period.__dict__, default=lambda o: o.__dict__)


def average_str(input_string, string_length):
    result = []
    if string_length < 0:
        return result
    input_len = len(input_string)
    for i in range(0, input_len, string_length):
        chunk = input_string[i:i+string_length]
        result.append(chunk)
    return result


def hex2decimal(hexinput):
    if not hexinput:
        return 0
    hexinput = re.sub("^0+", "", hexinput)
    if not hexinput:
        return 0
    return int(hexinput, 16)


import json
import colorsys
from .. import strategy


@strategy.register("hb_djv1_color")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(dp_value)
    return status_key, status_value


def convert_value(json_str):
    h, s, v = colorStrToHsvVOJson(json_str)
    rgb_map = hsv2rgbDecimalStandard(h, s, v)
    r = rgb_map["R"]
    g = rgb_map["G"]
    b = rgb_map["B"]
    return "|".join(map(str, [r, g, b]))


def colorStrToHsvVOJson(color_str):
    color_dict = json.loads(color_str)
    h = float(color_dict["h"]) / 360
    s = float(color_dict["s"]) / 255
    v = float(color_dict["v"]) / 255
    return h, s, v


def hsv2rgbDecimalStandard(h, s, v):
    rgb = colorsys.hsv_to_rgb(h, s, v)
    rgb_map = {
        "R": int(rgb[0] * 255),
        "G": int(rgb[1] * 255),
        "B": int(rgb[2] * 255),
    }
    return rgb_map


from enum import Enum

from .custom_strategy import custom_convert


class Strategy(object):
    def __init__(self):
        self._strategies = {}

    def register(self, name):
        def decorator(func):
            self._strategies[name] = func
            return func
        return decorator

    def convert(self, name, *args, **kwargs):
        if name not in self._strategies:
            raise Exception("Strategy {} not found.".format(name))

        # 标准指令集脚本转换
        return self._strategies.get(name)(*args, **kwargs)
        # 定制逻辑转换
        # _, config_item = args
        # status_item = custom_convert(status_item, config_item)
        # return status_item

# singleton
strategy = Strategy()


# 策略转换结束后的定制逻辑
# 定义枚举
class CMDCCodeEnum(Enum):
    switch = 1
    switchOne = 2
    switchTwo = 3
    switchThree = 4
    switchUsbOne = 5
    # ... (以此类推)

# 定义类
class CMDCStatusVO:
    def __init__(self, cmdcCodeEnum, type):
        self.cmdcCodeEnum = cmdcCodeEnum
        self.type = type

# 初始化字典
status2CMDCCodeMap = {}

status2CMDCCodeMap["switch"] = CMDCStatusVO(CMDCCodeEnum.switch, 1)
status2CMDCCodeMap["switch_1"] = CMDCStatusVO(CMDCCodeEnum.switchOne, 1)
# ... (以此类推)



if __name__ == "__main__":
    @strategy.register("f1")
    def f1():
        print("f1")

    @strategy.register("f2")
    def f2():
        print("f2")

    strategy.convert("f1")
    strategy.convert("f2")
    strategy.convert("f")


import json
from .. import strategy


@strategy.register("default")
def convert(dp_item: tuple, config_item: dict) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is not None and dp_value != "":
        return status_key, dp_value

    status_value = convert_value(config_item)
    return status_key, status_value


def convert_value(config_item):
    value_type = config_item["valueType"].capitalize()
    if value_type == "Boolean":
        status_value = False
    elif value_type == "Integer":
        status_value = json.loads(config_item["valueDesc"]).get("min")
    elif value_type == "Enum":
        status_value = json.loads(config_item["valueDesc"]).get("range")[0]
    elif value_type in ["String", "Raw", "Bitmap"]:
        status_value = ""
    else:
        status_value = ""
    return status_value



"""Tuya scene api."""

from types import SimpleNamespace
from typing import Any
from .customerapi import CustomerApi


class SharingScene(SimpleNamespace):
    """Smart Life Scene.
    Attributes:
        actions(list): scene actions
        enabled(bool): is scene enabled
        name(str): scene name
        scene_id(dict): scene id
        home_id(int): scene home id
    """

    actions: list
    enabled: bool
    name: str
    scene_id: str
    home_id: int


class SceneRepository:
    def __init__(self, customer_api: CustomerApi):
        self.api = customer_api

    def query_scenes(self, home_ids: list) -> list[SharingScene]:
        _scenes = []
        for home_id in home_ids:
            response = self.api.get("/v1.0/m/scene/ha/home/scenes", {"homeId": home_id})
            if response["success"]:
                for item in response["result"]:
                    scene = SharingScene(**item)
                    scene.home_id = home_id
                    _scenes.append(scene)

        return _scenes

    def trigger_scene(self, home_id: str, scene_id: str):
        response = self.api.post("/v1.0/m/scene/ha/trigger", None, {"homeId": home_id, "sceneId": scene_id})
        return response["result"]



import base64
import json

from .. import strategy


@strategy.register("db_v1_daily")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(decode4_to_hex_str(dp_value))
    return status_key, status_value


class DBV1DailyElectricDataVO:
    def __init__(self):
        self.startMonth = 0
        self.startDay = 0
        self.endMonth = 0
        self.endDay = 0
        self.electricTotal = 0.0

    def to_json(self):
        return json.dumps(self.__dict__)

def convert_value(data_str):
    vo = DBV1DailyElectricDataVO()
    vo.startMonth = hex_to_decimal(data_str[0:2])
    vo.startDay = hex_to_decimal(data_str[2:4])
    vo.endMonth = hex_to_decimal(data_str[4:6])
    vo.endDay = hex_to_decimal(data_str[6:8])
    vo.electricTotal = hex_to_decimal(data_str[8:16]) / 100.0
    return vo.to_json()

def decode(input_str):
    return base64.b64decode(input_str)

def decode4_to_hex_str(input_str):
    bytes_data = decode(input_str)
    return ''.join('{:02x}'.format(x) for x in bytes_data)

def hex_to_decimal(hex_str):
    hex_str = hex_str.lstrip('0')
    return int(hex_str or "0", 16)


"""device api."""
from __future__ import annotations

from types import SimpleNamespace
from typing import Any, Optional
import time

from .customerapi import CustomerApi
from .customerlogging import logger


class DeviceFunction(SimpleNamespace):
    """device's function.

    Attributes:
        code(str): function's code
        desc(str): function's description
        name(str): function's name
        type(str): function's type, which may be Boolean, Integer, Enum, Json
        values(dict): function's value range
    """

    code: str
    desc: str
    name: str
    type: str
    values: dict[str, Any]


class DeviceStatusRange(SimpleNamespace):
    """device's status range.

    Attributes:
        code(str): status's code
        type(str): status's type, which may be Boolean, Integer, Enum, Json
        values(dict): status's value range
    """

    code: str
    type: str
    values: str


class CustomerDevice(SimpleNamespace):
    """Customer Device.

    Attributes:
          id: Device id
          name: Device name
          local_key: Key
          category: Product category
          product_id: Product ID
          product_name: Product name
          sub: Determine whether it is a sub-device, true-> yes; false-> no
          uuid: The unique device identifier
          asset_id: asset id of the device
          online: Online status of the device
          icon: Device icon
          ip: Device IP
          time_zone: device time zone
          active_time: The last pairing time of the device
          create_time: The first network pairing time of the device
          update_time: The update time of device status

          status: Status set of the device
          function: Instruction set of the device
          status_range: Status value range set of the device
    """

    id: str
    name: str
    local_key: str
    category: str
    product_id: str
    product_name: str
    sub: bool
    uuid: str
    asset_id: str
    online: bool
    icon: str
    ip: str
    time_zone: str
    active_time: int
    create_time: int
    update_time: int
    set_up: Optional[bool] = False
    support_local: Optional[bool] = False
    local_strategy: dict[int, dict[str, Any]] = {}

    status: dict[str, Any] = {}
    function: dict[str, DeviceFunction] = {}
    status_range: dict[str, DeviceStatusRange] = {}

    def __eq__(self, other):
        """If devices are the same one."""
        return self.id == other.id


class DeviceRepository:
    def __init__(self, customer_api: CustomerApi):
        self.api = customer_api
        self.filter = Filter(10)

    def query_devices_by_home(self, home_id: str) -> list[CustomerDevice]:
        response = self.api.get(f"/v1.0/m/life/ha/home/devices", {"homeId": home_id})
        return self._query_devices(response)

    def query_devices_by_ids(self, ids: list) -> list[CustomerDevice]:
        response = self.api.get("/v1.0/m/life/ha/devices/detail", {"devIds": ",".join(ids)})
        return self._query_devices(response)

    def _query_devices(self, response) -> list[CustomerDevice]:
        _devices = []
        if response["success"]:
            for item in response["result"]:
                device = CustomerDevice(**item)
                status = {}
                for item_status in device.status:
                    if "code" in item_status and "value" in item_status:
                        code = item_status["code"]
                        value = item_status["value"]
                        status[code] = value
                device.status = status
                self.update_device_specification(device)
                self.update_device_strategy_info(device)
                _devices.append(device)
        return _devices

    def update_device_specification(self, device: CustomerDevice):
        device_id = device.id
        response = self.api.get(f"/v1.1/m/life/{device_id}/specifications")
        if response.get("success"):
            result = response.get("result", {})
            function_map = {}
            for function in result["functions"]:
                code = function["code"]
                function_map[code] = DeviceFunction(**function)

            status_range = {}
            for status in result["status"]:
                code = status["code"]
                status_range[code] = DeviceStatusRange(**status)

            device.function = function_map
            device.status_range = status_range

    def update_device_strategy_info(self, device: CustomerDevice):
        device_id = device.id
        response = self.api.get(f"/v1.0/m/life/devices/{device_id}/status")
        support_local = True
        if response.get("success"):
            result = response.get("result", {})
            pid = result["productKey"]
            dp_id_map = {}
            for dp_status_relation in result["dpStatusRelationDTOS"]:
                if not dp_status_relation["supportLocal"]:
                    support_local = False
                    break
                # statusFormat valueDesc、valueType,enumMappingMap,pid
                dp_id_map[dp_status_relation["dpId"]] = {
                    "value_convert": dp_status_relation["valueConvert"],
                    "status_code": dp_status_relation["statusCode"],
                    "config_item": {
                        "statusFormat": dp_status_relation["statusFormat"],
                        "valueDesc": dp_status_relation["valueDesc"],
                        "valueType": dp_status_relation["valueType"],
                        "enumMappingMap": dp_status_relation["enumMappingMap"],
                        "pid": pid,
                    }
                }
            device.support_local = support_local
            if support_local:
                device.local_strategy = dp_id_map

            logger.debug(
                f"device status strategy dev_id = {device_id} support_local = {support_local} local_strategy = {dp_id_map}")

    def send_commands(self, device_id: str, commands: list[dict[str, Any]]):
        if self.filter.call(device_id, commands):
            self.api.post(f"/v1.1/m/thing/{device_id}/commands", None, {"commands": commands})


class Filter:
    def __init__(self, time: int):
        self.last_call_time = {}
        self.time_limit = time
        self.last_clean_time = 0

    def clean_expired_keys(self):
        current_time = time.time()
        if current_time - self.last_clean_time >= 10:
            expired_keys = [key for key, (_, last_time) in self.last_call_time.items() if
                            current_time - last_time >= 10]
            for key in expired_keys:
                del self.last_call_time[key]
            self.last_clean_time = current_time

    def call(self, dev_id, param) -> bool:
        self.clean_expired_keys()

        current_time = time.time()
        if dev_id in self.last_call_time:
            last_param, last_time = self.last_call_time[dev_id]
            if param != last_param or current_time - last_time >= self.time_limit:
                self.last_call_time[dev_id] = (param, current_time)
                logger.debug(f"filter receive one dev_id = {dev_id} param = {param}")
                return True
            else:
                logger.debug(f"filter receive two dev_id = {dev_id} param = {param}")
                return False
        else:
            self.last_call_time[dev_id] = (param, current_time)
            logger.debug(f"filter receive three dev_id = {dev_id} param = {param}")
            return True



import json

from .. import strategy


@strategy.register("dj_v2_contr_alg")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(dp_value)
    return status_key, status_value

class DJV2ControlDataVO:
    def __init__(self):
        self.change_mode = ""
        self.h = 0
        self.s = 0
        self.v = 0
        self.bright = 0
        self.temperature = 0


def convert_value(input_str):
    vo = DJV2ControlDataVO()
    if not input_str:
        return ""
    flag = hex2decimal(input_str[:1])
    if flag == 0:
        vo.change_mode = "direct"
    if flag == 1:
        vo.change_mode = "gradient"
    vo.h = hex2decimal(input_str[1:5])
    vo.s = hex2decimal(input_str[5:9])
    vo.v = hex2decimal(input_str[9:13])
    vo.bright = hex2decimal(input_str[13:17])
    vo.temperature = hex2decimal(input_str[17:])
    return json.dumps(vo.__dict__)


def hex2decimal(hex_str: str) -> int:
    if not hex_str:
        return 0
    hex_str = hex_str.lstrip("0")
    if not hex_str:
        return 0
    return int(hex_str, 16)







import json
from enum import Enum

class CMDCCodeEnum(Enum):
    switch = 1
    switchOne = 2
    switchTwo = 3
    switchThree = 4
    switchUsbOne = 5
    switchUsbTwo = 6
    switchUsbThree = 7
    switchLed = 9
    switchSound = 10
    switchSpray = 11
    countDown = 13
    change_mode = 14
    color = 15
    brightness = 17
    temp = 18
    mode = 19
    ledMode = 20
    lowOil = 21

class CMDCStatusVO:
    def __init__(self, cmdcCodeEnum, type):
        self.cmdcCodeEnum = cmdcCodeEnum
        self.type = type

status2CMDCCodeMap = {
    "switch": CMDCStatusVO(CMDCCodeEnum.switch, 1),
    "switch_1": CMDCStatusVO(CMDCCodeEnum.switchOne, 1),
    "switch_2": CMDCStatusVO(CMDCCodeEnum.switchTwo, 1),
    "switch_3": CMDCStatusVO(CMDCCodeEnum.switchThree, 1),
    "switch_usb1": CMDCStatusVO(CMDCCodeEnum.switchUsbOne, 1),
    "switch_usb2": CMDCStatusVO(CMDCCodeEnum.switchUsbTwo, 1),
    "switch_usb3": CMDCStatusVO(CMDCCodeEnum.switchUsbThree, 1),
    "switch_led": CMDCStatusVO(CMDCCodeEnum.switch, 1),
    "colour_data": CMDCStatusVO(CMDCCodeEnum.color, 4),
    "colour_data_v2": CMDCStatusVO(CMDCCodeEnum.color, 4),
    "bright_value": CMDCStatusVO(CMDCCodeEnum.brightness, 3),
    "bright_value_v2": CMDCStatusVO(CMDCCodeEnum.brightness, 3),
    "temp_value": CMDCStatusVO(CMDCCodeEnum.temp, 3),
    "temp_value_v2": CMDCStatusVO(CMDCCodeEnum.temp, 3),
    "switch_spray": CMDCStatusVO(CMDCCodeEnum.switchSpray, 1),
    "mode": CMDCStatusVO(CMDCCodeEnum.mode, 2),
    "countdown": CMDCStatusVO(CMDCCodeEnum.countDown, 3),
    "moodlighting": CMDCStatusVO(CMDCCodeEnum.ledMode, 2),
    "switch_sound": CMDCStatusVO(CMDCCodeEnum.switchSound, 1)
}


def custom_convert(status_item, config_item):
    status_code, status_value = status_item
    if status_code not in status2CMDCCodeMap:
        return status_item

    cmdc_status_vo = status2CMDCCodeMap.get(status_code)
    status_code = cmdc_status_vo.cmdcCodeEnum.name
    if cmdc_status_vo.type == 1: # Boolean 类型
        if status_code == "switch_1" and config_item["pid"] == "of3pvbtfmg5jdw7o":
            status_code = CMDCCodeEnum.switch.name
            status_value = 1 if status_value == True or status_value == "true" or status_value == 1 else 0
    elif cmdc_status_vo.type == 2: # Enum 类型
        if status_code == "moodlighting":
            if is_integer(str(status_value)):
                status_value = int(status_value)
            else:
                return status_item
        elif status_code == "mode":
            if str(status_value) == "large":
                status_value = 2
            elif str(status_value) == "middle":
                status_value = 1
            elif str(status_value) == "small":
                status_value = 0
            elif str(status_value) == "interval":
                status_value = 3
            elif str(status_value) == "continuous":
                status_value = 4
            else:
                return status_item
        else:
            return status_item
    elif cmdc_status_vo.type == 3: # 数值类型
        if status_code == "bright_value":
            status_value = get_common_brightness_value(status_value, 25, 255)
        elif status_code == "bright_value_v2":
            status_value = get_common_brightness_value(status_value, 10, 1000)
        elif status_code == "temp_value":
            if is_integer(str(status_value)):
                status_value = int(status_value) * 100 / 255
            else:
                return status_item
        elif status_code == "temp_value_v2":
            if is_integer(str(status_value)):
                status_value = int(status_value) / 10
            else:
                return status_item
        elif status_code == "countdown":
            if str(status_value) == "cancel":
                status_value = 0
            elif is_integer(str(status_value)):
                status_value = str(int(status_value) * 60)
            else:
                return status_item
        else:
            return status_item
    elif cmdc_status_vo.type == 4: # 特殊类型
        if status_code == "colour_data" or status_code == "colour_data_v2":
            if status_value is None:
                return status_item
            else:
                if status_code == "colour_data" or status_code == "colour_data_v2":
                    skill_vo = get_skill_vo(config_item["pid"], status_item)
                    status_value = skill_vo.value
                    my_json = json.loads(str(status_value))
                    v = int(my_json["v"])
                    if status_code == "colour_data":
                        status_value = (v * 100 / 255) if (v * 100 % 255 == 0) else (v * 100 / 255 + 1)
                    else:
                        status_value = v / 10
                else:
                    pass
        else:
            return status_item
    return status_code, status_value


def get_common_brightness_value(value, min, max):
    try:
        temp_value = int(value)
    except ValueError:
        return None

    if temp_value == max:
        return 100
    elif temp_value == min:
        return min
    else:
        return (temp_value - 25) * 100 / 255


def is_integer(s):
    try:
        int(s)
        return True
    except ValueError:
        return False


class SkillCodeEnum(Enum):
    Color = {"type": "Enum", "desc": "颜色", "descEn": "Color"}


class SkillItemVO:
    def __init__(self, skill=None, value=None):
        self.skill = skill
        self.value = value


class SkillCodeVO:
    def __init__(self, skillCodeEnum=None, name=None, type=None, desc=None, descEn=None):
        if skillCodeEnum:
            self.name = skillCodeEnum.name
            self.type = None # Replace with appropriate conversion method for ValueType
            self.desc = skillCodeEnum.value["desc"]
            self.descEn = skillCodeEnum.value["descEn"]
        if name and type and desc and descEn:
            self.name = name
            self.type = type
            self.desc = desc
            self.descEn = descEn


def get_skill_item_vo(skill, value):
    return SkillItemVO(skill.name, value)


def get_skill_vo(pid, status_dto):
    status_code, status_value = status_dto
    if status_value is None:
        return None
    if str(status_value).startswith("{"):
        try:
            jsonObject = json.loads(str(status_value))
            h = jsonObject["h"]
            s = jsonObject["s"]
            v = jsonObject["v"]
            if ((s < 30 and 221 <= v <= 255) and status_code == 'colour_data') or \
                ((s < 227 and 866 <= v <= 1000) and status_code == 'colour_data_v2') or \
                ((s < 60 and 221 <= v <= 255) and status_code == 'colour_data_hsv'):
                return get_skill_item_vo(SkillCodeVO(SkillCodeEnum.Color), "white")
            else:
                if   0 <= h <= 5 or 358 <= h <= 360: color = "red"
                elif 6 <= h <= 16: color = "orange"
                elif 17 <= h <= 26: color = "yellow"
                elif 27 <= h <= 32: color = "golden"
                elif 33 <= h <= 68: color = "yellow"
                elif 69 <= h <= 133: color = "green"
                elif 134 <= h <= 208: color = "cyan"
                elif 209 <= h <= 260: color = "blue"
                elif 261 <= h <= 316: color = "purple"
                elif 317 <= h <= 357: color = "pink"
                else: return None
                return get_skill_item_vo(SkillCodeVO(SkillCodeEnum.Color), color)
        except Exception as e:
            print("Color e: ", e, ", pid: ", pid, ", value: ", status_value) # Replace with your logging mechanism
    elif is_integer(str(status_value)):
        num = int(status_value)
        if   0 <= num <= 3: color = "red"
        elif 4 <= num <= 6: color = "orange"
        elif 7 <= num <= 27: color = "pink"
        elif 28 <= num <= 60: color = "purple"
        elif 61 <= num <= 80: color = "blue"
        elif 81 <= num <= 125: color = "cyan"
        elif 126 <= num <= 233: color = "green"
        elif 234 <= num <= 249: color = "yellow"
        elif 250 <= num <= 255: color = "golden"
        else: return None
        return get_skill_item_vo(SkillCodeVO(SkillCodeEnum.Color), color)
    return get_skill_item_vo(SkillCodeVO(SkillCodeEnum.Color), status_value)


import base64
import json
from typing import Optional

from .. import strategy


@strategy.register("db_v1_params")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(decode4hex_str(dp_value))
    return status_key, status_value

def decode(input: str) -> bytes:
    return base64.b64decode(input)


def decode4hex_str(input: str) -> str:
    bytes_data = decode(input)
    return ''.join(f'{byte:02x}' for byte in bytes_data)


def convert_value(s: str) -> str:
    vo = DBV1CircuitParamsVO()
    vo.voltage = hex2decimal(s[0:4]) / 10.0
    vo.electricCurrent = hex2decimal(s[4:10]) / 1000.0
    vo.power = hex2decimal(s[10:16]) / 1000.0
    return json.dumps(vo.__dict__)


def hex2decimal(hex_str: str) -> int:
    if not hex_str:
        return 0
    hex_str = hex_str.lstrip('0')
    if not hex_str:
        return 0
    return int(hex_str, 16)


class DBV1CircuitParamsVO:
    def __init__(self, voltage: Optional[float] = None, electric_current: Optional[float] = None,
                 power: Optional[float] = None):
        self.voltage = voltage
        self.electricCurrent = electric_current
        self.power = power


"""Customer API."""
from __future__ import annotations

from typing import Any
import requests
from .customerlogging import logger
import json
import hmac
import hashlib
import random
import base64
from cryptography.hazmat.primitives.ciphers.aead import AESGCM
import uuid
from abc import ABCMeta

import time


class CustomerTokenInfo:
    """CUstomer token info.

    Attributes:
        access_token: Access token.
        expire_time: Valid period in seconds.
        refresh_token: Refresh token.
    """

    def __init__(self, token_info: dict[str, Any] = None):
        self.expire_time = (
                token_info.get("t", 0)
                + token_info.get("expire_time", 0) * 1000
        )
        self.uid = token_info.get("uid", "")
        self.access_token = token_info.get("access_token", "")
        self.refresh_token = token_info.get("refresh_token", "")


class CustomerApi:

    def __init__(
            self,
            token_info: CustomerTokenInfo,
            client_id: str,
            user_code: str,
            end_point: str,
            listener: SharingTokenListener
    ):
        self.session = requests.session()
        self.token_info = token_info
        self.client_id = client_id
        self.user_code = user_code
        self.endpoint = end_point
        self.refresh_token = False
        self.token_listener = listener

    def __request(
            self,
            method: str,
            path: str,
            params: dict[str, Any] | None = None,
            body: dict[str, Any] | None = None,
    ) -> dict[str, Any] | None:

        self.refresh_access_token_if_need()

        rid = str(uuid.uuid4())
        sid = ""
        md5 = hashlib.md5()
        rid_refresh_token = rid + self.token_info.refresh_token
        md5.update(rid_refresh_token.encode('utf-8'))
        hash_key = md5.hexdigest()
        secret = _secret_generating(rid, sid, hash_key)

        query_encdata = ""
        if params is not None and len(params.keys()) > 0:
            query_encdata = _form_to_json(params)
            query_encdata = _aes_gcm_encrypt(query_encdata, secret)
            params = {
                "encdata": query_encdata
            }
            query_encdata = str(query_encdata, encoding="utf8")
        body_encdata = ""
        if body is not None and len(body.keys()) > 0:
            body_encdata = _form_to_json(body)
            body_encdata = _aes_gcm_encrypt(body_encdata, secret)
            body = {
                "encdata": str(body_encdata, encoding="utf8")
            }
            body_encdata = str(body_encdata, encoding="utf8")

        t = int(time.time() * 1000)
        headers = {
            "X-appKey": self.client_id,
            "X-requestId": rid,
            "X-sid": sid,
            "X-time": str(t),
        }
        if self.token_info is not None and len(self.token_info.access_token) > 0:
            headers["X-token"] = self.token_info.access_token

        sign = _restful_sign(hash_key,
                             query_encdata,
                             body_encdata,
                             headers)
        headers["X-sign"] = sign

        response = self.session.request(
            method, self.endpoint + path, params=params, json=body, headers=headers
        )

        if response.ok is False:
            logger.error(
                f"Response error: code={response.status_code}, content={response.content}"
            )
            return None

        ret = response.json()
        logger.debug("response before decrypt ret = %s", ret)

        if not ret.get("success"):
            raise Exception(f"network error:({ret['code']}) {ret['msg']}")

        result = _aex_gcm_decrypt(ret.get("result"), secret)
        try:
            ret["result"] = json.loads(result)
        except json.decoder.JSONDecodeError:
            ret["result"] = result

        logger.debug("response ret = %s", ret)
        return ret

    def refresh_access_token_if_need(self):

        if self.refresh_token:
            return

        now = int(time.time() * 1000)
        expired_time = self.token_info.expire_time

        if expired_time - 60 * 1000 > now:  # 1min
            return

        self.refresh_token = True
        try:
            response = self.get("/v1.0/m/token/" + self.token_info.refresh_token)

            if response.get("success"):
                result = response.get("result", {})
                token_info = {
                    "t": response["t"],
                    "expire_time": result["expireTime"],
                    "uid": result["uid"],
                    "access_token": result["accessToken"],
                    "refresh_token": result["refreshToken"]
                }
                self.token_info = CustomerTokenInfo(token_info)
                if self.token_listener is not None:
                    self.token_listener.update_token(token_info)
        except Exception as e:
            logger.error("net work error = %s", e)
        finally:
            self.refresh_token = False

    def get(self, path: str, params: dict[str, Any] | None = None) -> dict[str, Any]:
        """Http Get.

        Requests the server to return specified resources.

        Args:
            path (str): api path
            params (map): request parameter

        Returns:
            response: response body
        """
        return self.__request("GET", path, params, None)

    def post(self, path: str, params: dict[str, Any] | None = None, body: dict[str, Any] | None = None) -> dict[
        str, Any]:
        """Http Post.

        Requests the server to update specified resources.

        Args:
            path (str): api path
            params (map): request parameter
            body (map): request body

        Returns:
            response: response body
        """
        return self.__request("POST", path, params, body)

    def put(self, path: str, body: dict[str, Any] | None = None) -> dict[str, Any]:
        """Http Put.

        Requires the server to perform specified operations.

        Args:
            path (str): api path
            body (map): request body

        Returns:
            response: response body
        """
        return self.__request("PUT", path, None, body)

    def delete(self, path: str, params: dict[str, Any] | None = None) -> dict[str, Any]:
        """Http Delete.

        Requires the server to delete specified resources.

        Args:
            path (str): api path
            params (map): request param

        Returns:
            response: response body
        """
        return self.__request("DELETE", path, params, None)


def _random_nonce(e=32):
    t = "ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678"
    a = len(t)
    n = ""
    for i in range(e):
        n += t[random.randint(0, a - 1)]
    return n


def _form_to_json(content: dict[str, Any] | None = None) -> str:
    return json.dumps(content, separators=(',', ':'))


def _aes_gcm_encrypt(raw_data: str, secret: str):
    nonce = _random_nonce(12)
    raw_data = raw_data.encode('utf-8')
    secret = secret.encode('utf-8')
    nonce = nonce.encode('utf-8')
    cipher = AESGCM(secret)
    ciphertext = cipher.encrypt(nonce, raw_data, None)

    return base64.b64encode(nonce) + base64.b64encode(ciphertext)


def _aex_gcm_decrypt(cipher_data: str, secret: str) -> str:
    cipher_data = base64.b64decode(cipher_data)
    nonce = cipher_data[:12]
    cipher_text = cipher_data[12:]
    secret = secret.encode('utf-8')
    cipher = AESGCM(secret)
    decrypt = cipher.decrypt(nonce, cipher_text, None)
    return str(decrypt, encoding="utf8")


def _secret_generating(rid, sid, hash_key) -> str:
    message = hash_key
    mod = 16
    if sid != "":
        sid_length = len(sid)
        length = sid_length if sid_length < mod else mod
        ecode = ""
        for i in range(length):
            idx = ord(sid[i]) % mod
            ecode += sid[idx]
        message += "_"
        message += ecode

    if isinstance(message, str):
        message = message.encode('utf-8')
    if isinstance(rid, str):
        rid = rid.encode('utf-8')

    checksum = hmac.new(rid, message, hashlib.sha256)
    byte_temp = checksum.digest()
    secret = byte_temp.hex()

    return secret[:16]


def _restful_sign(hash_key: str, query_encdata: str, body_encdata: str, data: dict[str, Any]) -> str:
    headers = ["X-appKey", "X-requestId", "X-sid", "X-time", "X-token"]
    header_sign_str = ""
    for item in headers:
        val = data.get(item, "")
        if val != "":
            header_sign_str += item + "=" + val + "||"

    sign_str = header_sign_str[:-2]

    if query_encdata is not None and query_encdata != "":
        sign_str += query_encdata
    if body_encdata is not None and body_encdata != "":
        sign_str += body_encdata

    sign_str = bytes(sign_str, 'utf-8')
    hash_key = bytes(hash_key, 'utf-8')

    hash_value = hmac.new(hash_key, sign_str, hashlib.sha256)
    return hash_value.hexdigest()


class SharingTokenListener(metaclass=ABCMeta):
    def update_token(self, token_info: dict[str, Any]):
        """Update token.
        """
        pass



import json
from .. import strategy


@strategy.register("sd_clean_record")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(dp_value)
    return status_key, status_value


def convert_value(string_value):
    length = len(string_value)
    if length == 6:
        return clean_time_and_area_resolve(string_value)
    elif length == 11:
        return clean_time_and_area_with_map_resolve(string_value)
    elif length == 18:
        return recored_and_clean_time_with_area_resolve(string_value)
    else:
        return all_property_resolve(string_value)

def clean_time_and_area_resolve(value):
    clean_time = int(value[:3])
    clean_area = int(value[3:6])
    data = {"record_time": "", "clean_time": clean_time, "clean_area": clean_area, "map_id": ""}
    return json.dumps(data)

def recored_and_clean_time_with_area_resolve(value):
    record_time = value[:12]
    clean_time = int(value[12:15])
    clean_area = int(value[15:18])
    data = {"record_time": record_time, "clean_time": clean_time, "clean_area": clean_area, "map_id": ""}
    return json.dumps(data)

def clean_time_and_area_with_map_resolve(value):
    clean_time = int(value[:3])
    clean_area = int(value[3:6])
    map_id = value[6:11]
    data = {"record_time": "", "clean_time": clean_time, "clean_area": clean_area, "map_id": map_id}
    return json.dumps(data)

def all_property_resolve(value):
    record_time = value[:12]
    clean_time = int(value[12:15])
    clean_area = int(value[15:18])
    map_id = value[18:23]
    data = {"record_time": record_time, "clean_time": clean_time, "clean_area": clean_area, "map_id": map_id}
    return json.dumps(data)


import json

from .. import strategy
from .default import convert_value as convert_default_value


@strategy.register("enum")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    """枚举策略转换规则

    根据 dp value 的值作为 key 从 enumMappingMap 中获取对应的 value 映射的标准指令值
    如果没有找到对应的 value 映射，则使用默认的转换规则
    :param dp_item: dp 值
    :param config_item: 转换规则
    :return: 标准指令值
    """
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    enum_mappings = config_item["enumMappingMap"]
    status_value = None
    if str(dp_value) in enum_mappings and "value" in enum_mappings[str(dp_value)]:
        status_value = enum_mappings[str(dp_value)]["value"]
    if status_value is None:
        status_value = convert_default_value(config_item)
    return status_key, status_value







from __future__ import annotations

from .customerapi import CustomerApi


class SmartLifeHome:
    def __init__(self, id: str, name: str):
        self.id = id
        self.name = name


class HomeRepository:
    def __init__(self, customer_api: CustomerApi):
        self.api = customer_api

    def query_homes(self) -> list[SmartLifeHome]:
        response = self.api.get(f"/v1.0/m/life/users/homes")

        if response.get("success", False):
            _homes = []
            for home in response["result"]:
                _home = SmartLifeHome(str(home["ownerId"]), home["name"])
                _homes.append(_home)
            return _homes

        return []



import json
from typing import List

from .. import strategy


@strategy.register("dj_v2_scene_alg")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(dp_value)
    return status_key, status_value

class DJV2SceneUnitVO:
    def __init__(self):
        self.unit_switch_duration = 0
        self.unit_gradient_duration = 0
        self.unit_change_mode = ""
        self.h = 0
        self.s = 0
        self.v = 0
        self.bright = 0
        self.temperature = 0

class DJV2SceneDataVO:
    def __init__(self):
        self.scene_num = 0
        self.scene_units = []

def hex2decimal(hex_str: str) -> int:
    if not hex_str:
        return 0
    hex_str = hex_str.lstrip("0")
    if not hex_str:
        return 0
    return int(hex_str, 16)

def convert_value(str: str) -> str:
    vo = DJV2SceneDataVO()
    vo.scene_num = 1 + hex2decimal(str[:2])
    scene_units = []
    unit_str = str[2:]
    num = len(unit_str) // 26
    for i in range(num):
        item = unit_str[i * 26:(i + 1) * 26]
        unit_vo = DJV2SceneUnitVO()
        unit_vo.unit_switch_duration = hex2decimal(item[:2])
        unit_vo.unit_gradient_duration = hex2decimal(item[2:4])
        flag = hex2decimal(item[4:6])
        if flag == 0:
            unit_vo.unit_change_mode = "static"
        elif flag == 1:
            unit_vo.unit_change_mode = "jump"
        elif flag == 2:
            unit_vo.unit_change_mode = "gradient"
        unit_vo.h = hex2decimal(item[6:10])
        unit_vo.s = hex2decimal(item[10:14])
        unit_vo.v = hex2decimal(item[14:18])
        unit_vo.bright = hex2decimal(item[18:22])
        unit_vo.temperature = hex2decimal(item[22:])
        scene_units.append(unit_vo)
    vo.scene_units = scene_units
    return json.dumps(vo.__dict__, default=lambda o: o.__dict__)







from __future__ import annotations

from typing import Any, Tuple, Dict

from .customerapi import CustomerApi
import requests

URL_PATH = "apigw.iotbing.com"


class LoginControl:
    def __init__(self):
        self.session = requests.session()

    def qr_code(self, client_id: str, schema: str, user_code: str) -> Dict[str, Any]:
        response = self.session.request("POST",
                                        f"https://{URL_PATH}/v1.0/m/life/home-assistant/qrcode/tokens?clientid={client_id}&usercode={user_code}&schema={schema}",
                                        params=None, json=None, headers=None)
        return response.json()

    def login_result(self, token: str, client_id: str, user_code: str) -> Tuple[bool, Dict[str, Any]]:
        response = self.session.request("GET",
                                        f"https://{URL_PATH}/v1.0/m/life/home-assistant/qrcode/tokens/{token}?clientid={client_id}&usercode={user_code}",
                                        params=None, json=None, headers=None)
        response = response.json()
        if response.get("success"):
            ret = response.get("result", {})
            ret["t"] = response.get("t")
            return True, ret

        return False, response


class UserRepository:
    def __init__(self, customer_api: CustomerApi):
        self.api = customer_api

    def unload(self, terminal_id: str):
        self.api.refresh_access_token_if_need()
        self.api.post("/v1.0/m/token/terminal/expire", None, {
            "accessToken": self.api.token_info.access_token,
            "terminalId": terminal_id
        })

    def user_version_report(self, system_version: str, ty_plugin_version: str, ty_sdk_version: str):
        self.api.post("/v1.0/m/life/home-assistant/qrcode/versions", None, {
            "system_version": system_version,
            "ty_plugin_version": ty_plugin_version,
            "ty_sdk_version": ty_sdk_version
        })



import logging

logger = logging.getLogger(__package__)

default_handler = logging.StreamHandler()
default_handler.setFormatter(
    logging.Formatter("[%(asctime)s] [customer-%(module)s] %(message)s")
)

logger.addHandler(default_handler)



import json
from typing import Optional

from .. import strategy


@strategy.register("dj_v2_color_alg")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(dp_value)
    return status_key, status_value


class DJV2ColourDataVO:
    def __init__(self, h: Optional[int] = None, s: Optional[int] = None, v: Optional[int] = None):
        self.h = h
        self.s = s
        self.v = v

    def __str__(self):
        return f"DJV2ColourDataVO{{h={self.h}, s={self.s}, v={self.v}}}"


def hex2decimal(hex_str: str) -> int:
    if not hex_str:
        return 0

    hex_str = hex_str.lstrip("0")

    if not hex_str:
        return 0

    return int(hex_str, 16)


def convert_value(str_input: str) -> str:
    vo = DJV2ColourDataVO()
    vo.h = hex2decimal(str_input[0:4])
    vo.s = hex2decimal(str_input[4:8])
    vo.v = hex2decimal(str_input[8:12])
    return json.dumps(vo.__dict__)



import json
import base64

from .. import strategy


@strategy.register("cz_timer2_alg")
def convert(dp_item: tuple, config_item: dict) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(numDecodeToIntList(dp_value))
    return status_key, status_value


def numDecodeToIntList(input):
    bytes = base64.b64decode(input)
    return [item & 0xFF for item in bytes]

def convert_value(input, mark = True):
    vos = []
    if mark:
        for i in range(0, len(input), 10):
            tempVO = {}
            tempVO["timer_switch"] = integer2bool(input[i])
            tempVO["week_day"] = int2weekArr2(input[i + 1])
            tempVO["start_time"] = intArr2Str(input[i + 2], input[i + 3])
            tempVO["end_time"] = intArr2Str(input[i + 4], input[i + 5])
            tempVO["open_time"] = intArr2Str(input[i + 6], input[i + 7])
            tempVO["close_time"] = intArr2Str(input[i + 8], input[i + 9])
            vos.append(tempVO)
    else:
        for i in range(0, len(input), 6):
            tempVO = {}
            tempVO["timer_switch"] = integer2bool(input[i])
            tempVO["week_day"] = int2weekArr2(input[i + 1])
            tempVO["start_time"] = intArr2Str(input[i + 2], input[i + 3])
            tempVO["end_time"] = intArr2Str(input[i + 4], input[i + 5])
            vos.append(tempVO)
    return json.dumps(vos)

def integer2bool(integer):
    return integer > 0

def int2weekArr2(input):
    binary_str = format(input, 'b')[::-1]
    week_days = [i for i, bit in enumerate(binary_str) if bit == '1']
    return week_days

def intArr2Str(high, low):
    res_int = high * 256 + low
    hour = str(res_int // 60).zfill(2)
    minute = str(res_int % 60).zfill(2)
    return f"{hour}:{minute}"


from .version import VERSION
from .manager import Manager, SharingDeviceListener
from .customerlogging import logger
from .device import CustomerDevice, DeviceFunction, DeviceStatusRange
from .scenes import SharingScene, SceneRepository
from .customerapi import CustomerApi, SharingTokenListener
from .user import LoginControl, UserRepository
from .strategy import strategy
from . import strategy_repo


__all__ = [
    "Manager",
    "logger",
    "CustomerDevice",
    "DeviceFunction",
    "DeviceStatusRange",
    "SharingScene",
    "CustomerApi",
    "SharingDeviceListener",
    "SharingTokenListener",
    "LoginControl",
    "SceneRepository",
    "UserRepository",
    "strategy"
]

__version__ = VERSION



"""Sharing Open IOT HUB which base on MQTT."""
from __future__ import annotations

import threading
from .customerapi import CustomerApi
from typing import Any, Callable
from requests.exceptions import RequestException
import time
from .customerlogging import logger
from .device import CustomerDevice
from paho.mqtt import client as mqtt
from urllib.parse import urlsplit
import json
import uuid

CONNECT_FAILED_NOT_AUTHORISED = 5


class SharingMQConfig:
    """Sharing mqtt config."""

    def __init__(self, mqConfigResponse: dict[str, Any] = {}) -> None:
        """Init SharingMQConfig."""
        result = mqConfigResponse.get("result", {})
        self.url = result.get("url", "")
        self.client_id = result.get("clientId", "")
        self.username = result.get("username", "")
        self.password = result.get("password", "")
        self.expire_time = result.get("expireTime", 0)
        self.owner_topic = result.get("topic").get("ownerId").get("sub")
        self.dev_topic = result.get("topic").get("devId").get("sub")


class SharingMQ(threading.Thread):
    def __init__(self, customer_api: CustomerApi, owner_ids: list, device: list[CustomerDevice]):
        super().__init__()
        self.api = customer_api
        self._stop_event = threading.Event()
        self.client = None
        self.mq_config = None
        self.message_listeners = set()
        self.owner_ids = owner_ids
        self.device = device

    def _get_mqtt_config(self) -> SharingMQConfig:
        link_id = f"tuya-device-sharing-sdk-python.{uuid.uuid1()}"
        response = self.api.post("/v1.0/m/life/ha/access/config", None,
                                 {"linkId": link_id})
        if (response.get("success"), False) is False:
            raise Exception("get mqtt config error.")

        return SharingMQConfig(response)

    def _on_disconnect(self, client, userdata, rc):
        if rc != 0:
            logger.error(f"Unexpected disconnection.{rc}")
        else:
            logger.debug("disconnect")

    def _on_connect(self, mqttc: mqtt.Client, user_data: Any, flags, rc):
        logger.debug(f"connect flags->{flags}, rc->{rc}")
        if rc == 0:
            for owner_id in self.owner_ids:
                mqttc.subscribe(self.mq_config.owner_topic.format(ownerId=owner_id))
            batch_size = 20
            for i in range(0, len(self.device), batch_size):
                batch_devices = self.device[i:i + batch_size]
                topics_to_subscribe = []
                for dev in batch_devices:
                    dev_id = dev.id
                    topic_str = self.subscribe_topic(dev_id, dev.support_local)
                    topics_to_subscribe.append((topic_str, 0))  # 指定主题和qos=0

                if topics_to_subscribe:
                    mqttc.subscribe(topics_to_subscribe)

        elif rc == CONNECT_FAILED_NOT_AUTHORISED:
            self.__run_mqtt()

    def subscribe_device(self, dev_id: str, device: CustomerDevice):
        self.device.append(device)
        topic = self.subscribe_topic(dev_id, device.support_local)
        self.client.subscribe(topic)

    def un_subscribe_device(self, dev_id: str, support_local: bool):
        topic = self.subscribe_topic(dev_id, support_local)
        self.client.unsubscribe(topic)

    def subscribe_topic(self, dev_id: str, support_local: bool) -> str:
        subscribe_topic = self.mq_config.dev_topic.format(devId=dev_id)
        if support_local:
            subscribe_topic += "/pen"
        else:
            subscribe_topic += "/sta"
        return subscribe_topic

    def _on_message(self, mqttc: mqtt.Client, user_data: Any, msg: mqtt.MQTTMessage):
        logger.debug(f"payload-> {msg.payload}")

        msg_dict = json.loads(msg.payload.decode("utf8"))

        logger.debug(f"on_message: {msg_dict}")

        for listener in self.message_listeners:
            listener(msg_dict)

    def _on_subscribe(self, mqttc: mqtt.Client, user_data: Any, mid, granted_qos):
        logger.debug(f"_on_subscribe: {mid}")

    def _on_log(self, mqttc: mqtt.Client, user_data: Any, level, string):
        logger.debug(f"_on_log: {string}")

    def run(self):
        """Method representing the thread's activity which should not be used directly."""
        backoff_seconds = 1
        while not self._stop_event.is_set():
            try:
                self.__run_mqtt()
                backoff_seconds = 1

                # reconnect every 2 hours required.
                time.sleep(self.mq_config.expire_time - 60)
            except RequestException as e:
                logger.exception(e)
                logger.error(f"failed to refresh mqtt server, retrying in {backoff_seconds} seconds.")

                time.sleep(backoff_seconds)
                backoff_seconds = min(backoff_seconds * 2, 60)  # Try at most every 60 seconds to refresh

    def __run_mqtt(self):
        mq_config = self._get_mqtt_config()

        self.mq_config = mq_config

        logger.debug(f"connecting {mq_config.url}")
        mqttc = self._start(mq_config)

        if self.client:
            self.client.disconnect()
        self.client = mqttc

    def _start(self, mq_config: SharingMQConfig) -> mqtt.Client:
        mqttc = mqtt.Client(mq_config.client_id)
        mqttc.username_pw_set(mq_config.username, mq_config.password)
        mqttc.user_data_set({"mqConfig": mq_config})
        mqttc.on_connect = self._on_connect
        mqttc.on_message = self._on_message
        mqttc.on_subscribe = self._on_subscribe
        mqttc.on_log = self._on_log
        mqttc.on_disconnect = self._on_disconnect

        url = urlsplit(mq_config.url)
        if url.scheme == "ssl":
            mqttc.tls_set()

        mqttc.connect(url.hostname, url.port)

        mqttc.loop_start()
        return mqttc

    def start(self):
        """Start mqtt.

        Start mqtt thread
        """
        logger.debug("start")
        super().start()

    def stop(self):
        """Stop mqtt.

        Stop mqtt thread
        """
        logger.debug("stop")
        self.message_listeners = set()
        try:
            self.client.disconnect()
        except Exception as e:
            logger.error("mq disconnect error %s", e)
        self.client = None
        self._stop_event.set()

    def add_message_listener(self, listener: Callable[[dict], None]):
        """Add mqtt message listener."""
        self.message_listeners.add(listener)

    def remove_message_listener(self, listener: Callable[[dict], None]):
        """Remvoe mqtt message listener."""
        self.message_listeners.discard(listener)



import colorsys
import json

from .. import strategy


@strategy.register("voice_atm_color")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(dp_value)
    return status_key, status_value

class DJHsvVO:
    def __init__(self, h=0, s=0, v=0):
        self.h = h
        self.s = s
        self.v = v

    def __str__(self):
        return f"DJHsvVO{{h={self.h}, s={self.s}, v={self.v}}}"

class HSVColorVO:
    def __init__(self, hue=0, saturation=0, brightness=0):
        self.hue = hue
        self.saturation = saturation
        self.brightness = brightness

def add_zero(s, length):
    while len(s) < length:
        s = "0" + s
    return s

def convert_value(str_colour_value):
    if str_colour_value[6:] == "0168ffff":
        r_s = str_colour_value[0:2]
        r_g = str_colour_value[2:4]
        r_b = str_colour_value[4:6]

        r = int(r_s, 16)
        g = int(r_g, 16)
        b = int(r_b, 16)

        hsv_map = rgb2hsv_standard(r, g, b)

        hsv_vo = DJHsvVO(
            h=round(hsv_map["H"] * 360, 1),
            s=round(hsv_map["S"] * 255, 1),
            v=round(hsv_map["V"] * 255, 1)
        )

        return json.dumps(hsv_vo.__dict__)

    vo = get_hsv_from_color_dp_value(str_colour_value)

    hsv_vo = DJHsvVO(
        h=vo.hue,
        s=round(vo.saturation * 255, 1),
        v=round(vo.brightness * 255, 1)
    )

    return json.dumps(hsv_vo.__dict__)

def rgb2hsv_standard(red, green, blue):
    hsv = colorsys.rgb_to_hsv(red / 255, green / 255, blue / 255)
    return {"H": hsv[0], "S": hsv[1], "V": hsv[2]}

def get_hsv_from_color_dp_value(dp_value):
    hh_hex = dp_value[6:8]
    hl_hex = dp_value[8:10]

    s_hex = dp_value[10:12]
    v_hex = dp_value[12:]

    hh_bin = add_zero(format(int(hh_hex, 16), "b"), 8)
    hl_bin = add_zero(format(int(hl_hex, 16), "b"), 8)

    h_string = hh_bin + hl_bin
    h = int(h_string, 2) / 1.0
    s = int(s_hex, 16) / 255
    v = int(v_hex, 16) / 255

    hsv_color_vo = HSVColorVO(
        hue=h,
        saturation=round(s, 4),
        brightness=round(v, 4)
    )

    return hsv_color_vo


import base64
import json
import math
from enum import Enum

from .. import strategy


@strategy.register("db_v1_alarm")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(decode4HexStr(dp_value))
    return status_key, status_value


def decode(input):
    return base64.b64decode(input)


def decode4HexStr(input):
    bytes = decode(input)
    return ''.join(format(x, '02x') for x in bytes)


def averageStr(inputString, length):
    if length < 0:
        return None

    return [inputString[i:i + length] for i in range(0, len(inputString), length)]


def hex2Decimal(hex):
    if not hex:
        return 0

    hex = hex.lstrip('0')
    return int(hex, 16) if hex else 0


class DBV1AlarmSetElectricEnum(Enum):
    OVERCURRENT = (1, "overcurrent", 0)
    THREE_PHASE_CURRENT_IMBALANCE = (2, "three_phase_current_imbalance", 0)
    AMMETER_OVERVOLTAGE = (3, "ammeter_overvoltage", 0)
    UNDER_VOLTAGE = (4, "under_voltage", 0)
    TREE_PHASE_CURRENT_LOSS = (5, "three_phase_current_loss", None)
    POWER_FAILURE = (6, "power_failure", None)
    MAGNETIC = (7, "magnetic", None)
    INSUFFICIENT_BALANCE = (8, "insufficient_balance", 0)
    ARREARS = (9, "arrears", None)
    BATTERY_OVERVOLTAGE = (10, "battery_overvoltage", 2)
    COVER_OPEN = (11, "cover_open", None)
    METER_COVER_OPEN = (12, "meter_cover_open", None)
    FAULT = (13, "fault", None)

    @staticmethod
    def getById(id):
        for alarmEnum in DBV1AlarmSetElectricEnum:
            if alarmEnum.value[0] == id:
                return alarmEnum
        return None


def convert_value(param):
    list = []
    stringList = averageStr(param, 8)
    for string in stringList:
        vo = {}
        alarmId = hex2Decimal(string[0:2])
        byId = DBV1AlarmSetElectricEnum.getById(alarmId)
        if byId is None:
            continue
        vo['alarmCode'] = byId.value[1]
        vo['doAction'] = hex2Decimal(string[2:4]) == 1
        if byId.value[2] is not None:
            threshold = hex2Decimal(string[4:8]) / math.pow(10, byId.value[2])
            if byId.value[2] > 0:
                vo['threshold'] = str(threshold)
            else:
                vo['threshold'] = str(int(threshold))
        list.append(vo)
    return json.dumps(list)


import json
from typing import Optional

from .. import strategy


@strategy.register("dj_v2_music_alg")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(dp_value)
    return status_key, status_value

class DJV2MusicDataVO:
    def __init__(self):
        self.change_mode: Optional[str] = None
        self.h: Optional[int] = None
        self.s: Optional[int] = None
        self.v: Optional[int] = None
        self.bright: Optional[int] = None
        self.temperature: Optional[int] = None

    def __str__(self):
        return f"DJV2MusicDataVO{{changeMode='{self.change_mode}', h={self.h}, s={self.s}, v={self.v}, bright={self.bright}, temperature={self.temperature}}}"


def convert_value(str_: str) -> str:
    vo = DJV2MusicDataVO()
    if not str_:
        return ""

    flag = hex2decimal(str_[0:1])
    if flag == 0:
        vo.change_mode = "direct"
    if flag == 1:
        vo.change_mode = "gradient"

    vo.h = hex2decimal(str_[1:5])
    vo.s = hex2decimal(str_[5:9])
    vo.v = hex2decimal(str_[9:13])
    vo.bright = hex2decimal(str_[13:17])
    vo.temperature = hex2decimal(str_[17:])

    return json.dumps(vo.__dict__)


def hex2decimal(hex_str: str) -> int:
    if not hex_str:
        return 0
    hex_str = hex_str.lstrip('0')
    if not hex_str:
        return 0
    return int(hex_str, 16)







import json

import base64

from .. import strategy


@strategy.register("db_v1_month")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(decode4hex_str(dp_value))
    return status_key, status_value

def decode(input: str) -> bytes:
    return base64.b64decode(input)

def decode4hex_str(input: str) -> str:
    bytes_data = decode(input)
    return ''.join(f'{byte:02x}' for byte in bytes_data)


class DBV1MonthElectricDataVO:
    def __init__(self, start_year=0, start_month=0, end_year=0, end_month=0, electric_total=0.0):
        self.startYear = start_year
        self.startMonth = start_month
        self.endYear = end_year
        self.endMonth = end_month
        self.electricTotal = electric_total

def hex2decimal(hex_str: str) -> int:
    if not hex_str:
        return 0
    hex_str = hex_str.lstrip("0")
    if not hex_str:
        return 0
    return int(hex_str, 16)

def convert_value(str: str) -> str:
    vo = DBV1MonthElectricDataVO()
    vo.startYear = hex2decimal(str[:2])
    vo.startMonth = hex2decimal(str[2:4])
    vo.endYear = hex2decimal(str[4:6])
    vo.endMonth = hex2decimal(str[6:8])
    vo.electricTotal = hex2decimal(str[8:16]) / 100.0
    return json.dumps(vo.__dict__)


import json
from typing import List, Dict
import colorsys
from .. import strategy

@strategy.register("dj_v1_scene_alg")
def convert(dp_item: tuple, config_item: dict) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(dp_value)
    return status_key, status_value


class DJHsvVO:
    def __init__(self, h: float = None, s: float = None, v: float = None):
        self.h = h
        self.s = s
        self.v = v

    def __repr__(self):
        return f"DJHsvVO(h={self.h}, s={self.s}, v={self.v})"

class DJV1SceneVO:
    def __init__(self, frequency: int = None, bright: int = None, temperature: int = None, hsv: List[DJHsvVO] = None):
        self.frequency = frequency
        self.bright = bright
        self.temperature = temperature
        self.hsv = hsv if hsv is not None else []

    def __repr__(self):
        return f"DJV1SceneVO(frequency={self.frequency}, bright={self.bright}, temperature={self.temperature}, hsv={self.hsv})"

def hex2decimal(hex_str: str) -> int:
    return int(hex_str, 16)

def split_str_by_width(s: str, width: int) -> List[str]:
    return [s[i:i+width] for i in range(0, len(s), width)]

def rgb2hsv_standard(r: int, g: int, b: int) -> Dict[str, float]:
    hsv = colorsys.rgb_to_hsv(r / 255.0, g / 255.0, b / 255.0)
    hsv_map = {"H": hsv[0], "S": hsv[1], "V": hsv[2]}
    return hsv_map

def scale_round(value: float, scale: int) -> float:
    return round(value, scale)

def convert_value(str_colour_value: str) -> str:
    if not str_colour_value:
        return ""

    vo = DJV1SceneVO()

    bright = hex2decimal(str_colour_value[0:2])
    temperature = hex2decimal(str_colour_value[2:4])
    frequency = hex2decimal(str_colour_value[4:6])
    hsv_len = hex2decimal(str_colour_value[6:8])

    vo.bright = bright
    vo.temperature = temperature
    vo.frequency = frequency

    hsv_str = str_colour_value[8:]
    rgb_str_list = split_str_by_width(hsv_str, 6)

    for rgb_str in rgb_str_list:
        r = hex2decimal(rgb_str[0:2])
        g = hex2decimal(rgb_str[2:4])
        b = hex2decimal(rgb_str[4:6])

        map = rgb2hsv_standard(r, g, b)

        h = scale_round(map["H"] * 360, 1)
        s = scale_round(map["S"] * 255, 1)
        v = scale_round(map["V"] * 255, 1)

        item_vo = DJHsvVO(h, s, v)

        vo.hsv.append(item_vo)

    return json.dumps(vo, default=lambda o: o.__dict__)


import json
from .. import strategy


@strategy.register("hb_jsq_lightv1")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, ""
    else:
        return status_key, dp_value + "|0|0"



from __future__ import annotations

from typing import Any, Literal, Optional

from .customerapi import CustomerApi, CustomerTokenInfo, SharingTokenListener
from .device import DeviceRepository, CustomerDevice
from .home import HomeRepository, SmartLifeHome
from .scenes import SceneRepository
from .user import UserRepository
from .strategy import strategy

from abc import ABCMeta, abstractclassmethod
from .customerlogging import logger
from .mq import SharingMQ
import time

PROTOCOL_DEVICE_REPORT = 4
PROTOCOL_OTHER = 20
BIZCODE_ONLINE = "online"
BIZCODE_OFFLINE = "offline"
BIZCODE_NAME_UPDATE = "nameUpdate"
BIZCODE_DPNAME_UPDATE = "dpNameUpdate"
BIZCODE_BIND_USER = "bindUser"
BIZCODE_DELETE = "delete"


class Manager:

    def __init__(
            self,
            client_id: str,
            user_code: str,
            terminal_id: str,
            end_point: str,
            token_response: dict[str, Any] = None,
            listener: SharingTokenListener = None,
    ) -> None:
        self.terminal_id = terminal_id
        self.customer_api = CustomerApi(
            CustomerTokenInfo(token_response),
            client_id,
            user_code,
            end_point,
            listener,
        )
        self.device_map: dict[str, CustomerDevice] = {}
        self.user_homes: list[SmartLifeHome] = []
        self.home_repository = HomeRepository(self.customer_api)
        self.device_repository = DeviceRepository(self.customer_api)
        self.device_listeners = set()

        self.mq = None
        self.scene_repository = SceneRepository(self.customer_api)
        self.user_repository = UserRepository(self.customer_api)

    def update_device_cache(self):
        self.device_map.clear()
        homes = self.home_repository.query_homes()
        self.user_homes = homes

        for home in homes:
            devices_by_home = self.device_repository.query_devices_by_home(home.id)
            for device in devices_by_home:
                self.device_map[device.id] = device

    def report_version(self, ha_version: str, integration_version: str, sdk_version: str):
        logger.debug(
            f"report version ha_version={ha_version},integration_version={integration_version},sdk_version={sdk_version}")
        self.user_repository.user_version_report(ha_version, integration_version, sdk_version)

    def _update_device_list_info_cache(self, ids: list[str]):
        devices = self.device_repository.query_devices_by_ids(ids)
        for device in devices:
            self.device_map[device.id] = device

    def refresh_mq(self):
        if self.mq is not None:
            self.mq.stop()
            self.mq = None

        home_ids = [home.id for home in self.user_homes]
        device = [device for device in self.device_map.values() if
                  hasattr(device, "id") and getattr(device, "set_up", False)]

        sharing_mq = SharingMQ(self.customer_api, home_ids, device)
        sharing_mq.start()
        sharing_mq.add_message_listener(self.on_message)
        self.mq = sharing_mq

    def send_commands(
            self, device_id: str, commands: list[dict[str, Any]]
    ):
        return self.device_repository.send_commands(device_id, commands)

    def get_device_stream_allocate(
            self, device_id: str, stream_type: Literal["flv", "hls", "rtmp", "rtsp"]
    ) -> Optional[str]:
        """Get the live streaming address by device ID and the video type.

        These live streaming video protocol types are available: RTSP, HLS, FLV, and RTMP.

        Args:
          device_id(str): device id
          stream_type(str): type of stream

        Returns:
            None or URL to the requested stream
        """
        response = self.customer_api.post(f"/v1.0/m/ipc/{device_id}/stream/actions/allocate", None,
                                          {"type": stream_type})
        if response["success"]:
            return response["result"]["url"]
        return None

    def query_scenes(self) -> list:
        """Query home scenes."""
        home_ids = [home.id for home in self.user_homes]
        return self.scene_repository.query_scenes(home_ids)

    def trigger_scene(self, home_id: str, scene_id: str):
        """Trigger home scene"""
        self.scene_repository.trigger_scene(home_id, scene_id)

    def on_message(self, msg: dict):
        logger.debug(f"mq receive-> {msg}")

        try:
            protocol = msg.get("protocol", 0)
            data = msg.get("data", {})

            if protocol == PROTOCOL_DEVICE_REPORT:
                self._on_device_report(data["devId"], data["status"])
            if protocol == PROTOCOL_OTHER and data['bizCode'] in [BIZCODE_DELETE, BIZCODE_BIND_USER,
                                                                  BIZCODE_DPNAME_UPDATE, BIZCODE_NAME_UPDATE,
                                                                  BIZCODE_OFFLINE, BIZCODE_ONLINE]:
                self._on_device_other(data["bizData"]["devId"], data["bizCode"], data)
        except Exception as e:
            logger.error("on message error = %s", e)

    def __update_device(
        self,
        device: CustomerDevice,
        updated_status_properties: list[str] | None = None,
        dp_timestamps: dict | None = None,
    ):
        for listener in self.device_listeners:
            listener.update_device(device, updated_status_properties, dp_timestamps)

    def _on_device_report(self, device_id: str, status: list):
        device = self.device_map.get(device_id, None)
        if not device:
            return
        logger.debug(f"mq _on_device_report-> {status}")
        updated_status_properties = []
        dp_timestamps = {}
        value = None
        if device.support_local:
            for item in status:
                # [{'dpId': 1, 't': 1752456620499, 'value': 120}]
                if "dpId" in item and "value" in item:
                    dp_id_item = device.local_strategy[item["dpId"]]
                    strategy_name = dp_id_item["value_convert"]
                    config_item = dp_id_item["config_item"]
                    dp_item = (dp_id_item["status_code"], item["value"])
                    logger.debug(
                        f"mq _on_device_report before strategy convert strategy_name={strategy_name},dp_item={dp_item},config_item={config_item}")
                    code, value = strategy.convert(strategy_name, dp_item, config_item)
                    logger.debug(f"mq _on_device_report after strategy convert code={code},value={value}")
                    device.status[code] = value
                    updated_status_properties.append(code)
                    if t := item.get("t"):
                        dp_timestamps[code] = t
        else:
            for item in status:
                if "code" in item and "value" in item:
                    code = item["code"]
                    value = item["value"]
                    device.status[code] = value
                    updated_status_properties.append(code)

        self.__update_device(device, updated_status_properties, dp_timestamps)

    def _on_device_other(self, device_id: str, biz_code: str, data: dict[str, Any]):
        logger.debug(f"mq _on_device_other-> {device_id} -- {biz_code}")

        # bind device to user
        if biz_code == BIZCODE_BIND_USER:
            device_ids = [device_id]
            # wait for es sync
            time.sleep(1)

            self._update_device_list_info_cache(device_ids)

            if device_id in self.device_map.keys():
                device = self.device_map.get(device_id)
                self.mq.subscribe_device(device_id, device)
                for listener in self.device_listeners:
                    listener.add_device(device)

        # device status update
        device = self.device_map.get(device_id, None)
        if not device:
            return

        if biz_code == BIZCODE_ONLINE:
            device.online = True
            self.__update_device(device)
        elif biz_code == BIZCODE_OFFLINE:
            device.online = False
            self.__update_device(device)
        elif biz_code == BIZCODE_NAME_UPDATE:
            device.name = data["bizData"]["name"]
            self.__update_device(device)
        elif biz_code == BIZCODE_DPNAME_UPDATE:
            pass
        elif biz_code == BIZCODE_DELETE:
            del self.device_map[device_id]
            self.mq.un_subscribe_device(device_id, device.support_local)
            for listener in self.device_listeners:
                listener.remove_device(device.id)

    def add_device_listener(self, listener: SharingDeviceListener):
        """Add device listener."""
        self.device_listeners.add(listener)

    def remove_device_listener(self, listener: SharingDeviceListener):
        """Remove device listener."""
        self.device_listeners.remove(listener)

    def unload(self):
        self.user_repository.unload(self.terminal_id)


class SharingDeviceListener(metaclass=ABCMeta):
    """Sharing device listener."""

    @abstractclassmethod
    def update_device(
        self,
        device: CustomerDevice,
        updated_status_properties: list[str] | None = None,
        dp_timestamps: dict | None = None,
    ) -> None:
        """Update device info.

        Args:
            device(CustomerDevice): updated device info
        """
        pass

    @abstractclassmethod
    def add_device(self, device: CustomerDevice):
        """Device Added.

        Args:
            device(CustomerDevice): Device added
        """
        pass

    @abstractclassmethod
    def remove_device(self, device_id: str):
        """Device removed.

        Args:
            device_id(str): device's id which removed
        """
        pass



import base64
import json
from .. import strategy


@strategy.register("ms_dp_syn_alg")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(dp_value)
    return status_key, status_value

def convert_value(input_str):
    res = set()
    if not input_str:
        return res
    try:
        data = input_str
        sync_byte = base64.b64decode(data)
        data_size = len(sync_byte)
        step = 2
        bit_size = 8
        index = 0
        while index < data_size:
            partion = (sync_byte[index] & 0b01111111) & 0xFF
            value = int(sync_byte[index + 1] & 0xFF)
            for i_bit in range(bit_size):
                if is_true(value, i_bit):
                    unlock_index = (partion - 1) * 8 + i_bit
                    res.add(unlock_index)
            index += step
    except Exception as e:
        print("MsAlg-门锁上报解析错误，上报数据协议非法. input_str=" + input_str)
        res = set()
    return res

def pow2(num):
    return int(2 ** num)

def is_true(number, position):
    decimal_value = pow2(position)
    return (decimal_value & number) == decimal_value


import json
from typing import Dict, Optional
from colorsys import rgb_to_hsv
from .. import strategy


@strategy.register("dj_v1_hsv_alg")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(dp_value)
    return status_key, status_value


class DJHsvVO:
    def __init__(self, h: Optional[float] = None, s: Optional[float] = None, v: Optional[float] = None):
        self.h = h
        self.s = s
        self.v = v

    def __str__(self):
        return f"DJHsvVO{{h={self.h}, s={self.s}, v={self.v}}}"


class HSVColorVO:
    def __init__(self, hue: float, saturation: float, brightness: float):
        self.hue = hue
        self.saturation = saturation
        self.brightness = brightness


def hex2decimal(hex_str: str) -> int:
    if not hex_str or len(hex_str) > 2:
        return 0
    return int(hex_str, 16)


def rgb2hsv_standard(red: int, green: int, blue: int) -> Dict[str, float]:
    hsv = rgb_to_hsv(red / 255, green / 255, blue / 255)
    return {"H": hsv[0], "S": hsv[1], "V": hsv[2]}


def scale_round(e: float, scale: int) -> float:
    return round(e, scale)


def get_hsv_from_color_dp_value(dp_value: str) -> HSVColorVO:
    hh_hex = dp_value[6:8]
    hl_hex = dp_value[8:10]
    s_hex = dp_value[10:12]
    v_hex = dp_value[12:]

    hh_bin = format(int(hh_hex, 16), "b")
    hl_bin = format(int(hl_hex, 16), "b")

    hh_bin = add_zero(hh_bin, 8)
    hl_bin = add_zero(hl_bin, 8)

    h_string = hh_bin + hl_bin
    h = int(h_string, 2) / 1.0
    s = int(s_hex, 16) * 1.0 / 255
    v = int(v_hex, 16) * 1.0 / 255

    hsv_color_vo = HSVColorVO(h, round(s, 3), round(v, 3))
    return hsv_color_vo


def add_zero(s: str, length: int) -> str:
    while len(s) < length:
        s = "0" + s
    return s


def convert_value(str_colour_value: str) -> str:
    if str_colour_value[6:] == "0168ffff":
        r = hex2decimal(str_colour_value[0:2])
        g = hex2decimal(str_colour_value[2:4])
        b = hex2decimal(str_colour_value[4:6])

        hsv_map = rgb2hsv_standard(r, g, b)

        hsv_vo = DJHsvVO()
        hsv_vo.h = scale_round(hsv_map["H"] * 360.0, 1)
        hsv_vo.s = scale_round(hsv_map["S"] * 255.0, 1)
        hsv_vo.v = scale_round(hsv_map["V"] * 255.0, 1)

        return json.dumps(hsv_vo.__dict__)

    vo = get_hsv_from_color_dp_value(str_colour_value)

    hsv_vo = DJHsvVO()
    hsv_vo.h = vo.hue
    hsv_vo.s = scale_round(vo.saturation * 255.0, 1)
    hsv_vo.v = scale_round(vo.brightness * 255.0, 1)

    return json.dumps(hsv_vo.__dict__)


from setuptools import find_packages, setup

from tuya_sharing import __version__


def requirements():
    with open("requirements.txt") as fileobj:
        return [line.strip() for line in fileobj]


with open("README.md", encoding="utf-8") as fh:
    doc_long_description = fh.read()


setup(
    name="tuya-device-sharing-sdk",
    url="https://github.com/tuya/tuya-device-sharing-sdk",
    author="Tuya Inc.",
    author_email="developer@tuya.com",
    keywords="tuya device sdk python",
    long_description=doc_long_description,
    long_description_content_type="text/markdown",
    description="A Python sdk for Tuya Open API, which provides IoT capabilities, maintained by Tuya official",
    license="MIT",
    project_urls={
        "Bug Tracker": "https://github.com/tuya/tuya-device-sharing-sdk/issues",
        "Changes": "https://github.com/tuya/tuya-device-sharing-sdk/wiki/Tuya-Device-Sharing-SDK-Release-Notes",
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    version=__version__,
    install_requires=requirements(),
    test_suite="runtests.runtests",
    entry_points={"nose.plugins": []},
    packages=find_packages(),
    python_requires=">=3.7",
)


import json
from .. import strategy


@strategy.register("hb_range_v1")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, ""

    status_value = convert_value(int(dp_value))
    return status_key, status_value

import math

def convert_value(input, t_min = 0, t_max = 100, i_min = 25, i_max = 255, mark = False):
    input_num = input
    if input_num > i_max:
        input_num = i_max
    if input_num < i_min:
        input_num = i_min

    num = (((t_max - t_min) * 1.0) / ((i_max - i_min) * 1.0)) * (input_num - i_min) + (t_min * 1.0)

    target = 0
    if mark:
        target = math.ceil(num)
    else:
        target = math.floor(num)

    if target > t_max:
        target = t_max
    if target < t_min:
        target = t_min
    return target


import json
import base64

from .. import strategy


@strategy.register("db_v1_frozen")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(decode4hex_str(dp_value))
    return status_key, status_value


def decode(input: str) -> bytes:
    return base64.b64decode(input)

def decode4hex_str(input: str) -> str:
    bytes_data = decode(input)
    return ''.join(f'{byte:02x}' for byte in bytes_data)

class DBV1FrozenTimeVO:
    def __init__(self, day=0, hour=0):
        self.day = day
        self.hour = hour

def hex2decimal(hex_str: str) -> int:
    if not hex_str:
        return 0
    hex_str = hex_str.lstrip("0")
    if not hex_str:
        return 0
    return int(hex_str, 16)

def convert_value(str: str) -> str:
    vo = DBV1FrozenTimeVO()
    vo.day = hex2decimal(str[:2])
    vo.hour = hex2decimal(str[2:4])
    return json.dumps(vo.__dict__)


from . import cz_timer1_alg
from . import cz_timer2_alg
from . import db_v1_alarm
from . import db_v1_daily
from . import db_v1_data
from . import db_v1_frozen
from . import db_v1_month
from . import db_v1_params
from . import db_v1_tariff
from . import default
from . import dj_v1_hsv_alg
from . import dj_v1_scene_alg
from . import dj_v2_color_alg
from . import dj_v2_contr_alg
from . import dj_v2_music_alg
from . import dj_v2_scene_alg
from . import enum_
from . import hb_djv1_color
from . import hb_jsq_lightv1
from . import hb_range_v1
from . import hb_range_v2
from . import ms_dp_syn_alg
from . import sd_clean_record
from . import voice_atm_color


import base64
import json
import re
from collections import namedtuple

from .. import strategy


@strategy.register("db_v1_data")
def convert(dp_item: tuple, config_item: dict = None) -> tuple:
    dp_key, dp_value = dp_item
    status_key, _ = json.loads(config_item["statusFormat"]).popitem()
    if dp_value is None:
        return status_key, dp_value

    status_value = convert_value(decode(dp_value))
    return status_key, status_value


def decode(input_str):
    return base64.b64decode(input_str)

DBV1PowerAndTimeVO = namedtuple('DBV1PowerAndTimeVO', ['power', 'year', 'month', 'date', 'hour', 'minute', 'second'])

def convert_value(str_input):
    res = DBV1PowerAndTimeVO(
        power = hex2dec(str_input[:8]) / 100.0,
        year = hex2dec(str_input[8:10]),
        month = hex2dec(str_input[10:12]),
        date = hex2dec(str_input[12:14]),
        hour = hex2dec(str_input[14:16]),
        minute = hex2dec(str_input[16:18]),
        second = hex2dec(str_input[18:20])
    )
    return json.dumps(res._asdict())

def average_str(input_string, length):
    if length < 0:
        return None
    res = []
    str_len = len(input_string)
    cur = 0
    while cur <= str_len:
        if cur + length > str_len:
            cur += length
            continue
        str_val = input_string[cur:cur+length]
        res.append(str_val)
        cur += length
    return res


def hex2dec(hex_str):
    if not hex_str:
        return 0
    hex_input = re.sub('^0+', '', hex_str)
    if not hex_input:
        return 0
    return int(hex_input, 16)